package net.knarcraft.blockhunt;

import me.libraryaddict.disguise.DisguiseAPI;
import me.libraryaddict.disguise.disguisetypes.DisguiseType;
import me.libraryaddict.disguise.disguisetypes.MiscDisguise;
import net.knarcraft.blockhunt.arena.Arena;
import net.knarcraft.blockhunt.arena.ArenaHandler;
import net.knarcraft.blockhunt.arena.ArenaState;
import net.knarcraft.blockhunt.config.MessageKey;
import net.knarcraft.blockhunt.listener.OnBlockBreakEvent;
import net.knarcraft.blockhunt.listener.OnBlockPlaceEvent;
import net.knarcraft.blockhunt.listener.OnEntityDamageByEntityEvent;
import net.knarcraft.blockhunt.listener.OnEntityDamageEvent;
import net.knarcraft.blockhunt.listener.OnFoodLevelChangeEvent;
import net.knarcraft.blockhunt.listener.OnInventoryClickEvent;
import net.knarcraft.blockhunt.listener.OnInventoryCloseEvent;
import net.knarcraft.blockhunt.listener.OnPlayerDropItemEvent;
import net.knarcraft.blockhunt.listener.OnPlayerInteractEvent;
import net.knarcraft.blockhunt.listener.OnPlayerMoveEvent;
import net.knarcraft.blockhunt.listener.OnPlayerQuitEvent;
import net.knarcraft.blockhunt.listener.OnPlayerSwapHandItemsEvent;
import net.knarcraft.blockhunt.listener.OnPlayerTeleportEvent;
import net.knarcraft.blockhunt.listener.OnSignChangeEvent;
import net.knarcraft.blockhunt.manager.CommandManager;
import net.knarcraft.blockhunt.manager.ConfigManager;
import net.knarcraft.blockhunt.manager.MessageManager;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.WordUtils;
import org.bukkit.Bukkit;
import org.bukkit.GameMode;
import org.bukkit.Location;
import org.bukkit.Material;
import org.bukkit.Sound;
import org.bukkit.World;
import org.bukkit.block.Block;
import org.bukkit.block.data.BlockData;
import org.bukkit.configuration.serialization.ConfigurationSerialization;
import org.bukkit.enchantments.Enchantment;
import org.bukkit.entity.Player;
import org.bukkit.event.Listener;
import org.bukkit.inventory.ItemStack;
import org.bukkit.plugin.PluginDescriptionFile;
import org.bukkit.plugin.java.JavaPlugin;

import java.util.ArrayList;

public class BlockHunt extends JavaPlugin implements Listener {

    private static PluginDescriptionFile pluginDescriptionFile;
    public static BlockHunt plugin;
    public String rootPermission;

    public void onEnable() {
        getServer().getPluginManager().registerEvents(this, this);

        getServer().getPluginManager().registerEvents(new OnBlockBreakEvent(), this);
        getServer().getPluginManager().registerEvents(new OnBlockPlaceEvent(), this);
        getServer().getPluginManager().registerEvents(new OnEntityDamageByEntityEvent(), this);
        getServer().getPluginManager().registerEvents(new OnEntityDamageEvent(), this);
        getServer().getPluginManager().registerEvents(new OnFoodLevelChangeEvent(), this);
        getServer().getPluginManager().registerEvents(new OnInventoryClickEvent(), this);
        getServer().getPluginManager().registerEvents(new OnInventoryCloseEvent(), this);
        getServer().getPluginManager().registerEvents(new OnPlayerSwapHandItemsEvent(), this);

        getServer().getPluginManager().registerEvents(new OnPlayerDropItemEvent(), this);
        getServer().getPluginManager().registerEvents(new OnPlayerInteractEvent(), this);
        getServer().getPluginManager().registerEvents(new OnPlayerMoveEvent(), this);
        getServer().getPluginManager().registerEvents(new OnPlayerTeleportEvent(), this);
        getServer().getPluginManager().registerEvents(new OnPlayerQuitEvent(), this);
        getServer().getPluginManager().registerEvents(new OnSignChangeEvent(), this);

        ConfigurationSerialization.registerClass(Arena.class, "BlockHuntArena");

        BlockHunt.pluginDescriptionFile = getDescription();
        this.rootPermission = BlockHunt.getPluginDescriptionFile().getName().toLowerCase() + ".";
        plugin = this;

        ConfigManager.newFiles();

        CommandManager.load();

        if (!getServer().getPluginManager().isPluginEnabled("LibsDisguises")) {
            MessageManager.broadcastFMessage(MessageKey.ERROR_LIBS_DISGUISES_NOT_INSTALLED);
            Bukkit.getServer().getPluginManager().disablePlugin(this);
            return;
        }

        if (!getServer().getPluginManager().isPluginEnabled("ProtocolLib")) {
            MessageManager.broadcastFMessage(MessageKey.ERROR_PROTOCOL_LIB_NOT_INSTALLED);
            Bukkit.getServer().getPluginManager().disablePlugin(this);
            return;
        }

        ArenaHandler.loadArenas();

        MessageManager.sendFMessage(null, MessageKey.LOG_ENABLED_PLUGIN, "name-" + BlockHunt.pluginDescriptionFile.getName(), "version-" + BlockHunt.pluginDescriptionFile.getVersion(), "authors-"
                + BlockHunt.pluginDescriptionFile.getAuthors().get(0));

        // Welcome to the massive game loop!!
        getServer().getScheduler().runTaskTimer(this, () -> {
            for (Arena arena : MemoryStorage.arenaList) {
                if (arena.gameState == ArenaState.WAITING) {
                    if (arena.playersInArena.size() >= arena.minPlayers) {
                        arena.gameState = ArenaState.STARTING;
                        arena.timer = arena.timeInLobbyUntilStart;
                        ArenaHandler.sendFMessage(arena, MessageKey.NORMAL_LOBBY_ARENA_IS_STARTING, "1-" + arena.timeInLobbyUntilStart);
                    }
                } else if (arena.gameState == ArenaState.STARTING) {
                    arena.timer = arena.timer - 1;
                    if (arena.timer > 0) {
                        if (arena.timer == 60 || arena.timer == 30 || arena.timer == 10) {
                            ArenaHandler.sendFMessage(arena, MessageKey.NORMAL_LOBBY_ARENA_IS_STARTING, "1-" + arena.timer);
                        } else if (arena.timer <= 5) {
                            World world = arena.lobbyWarp.getWorld();
                            if (world != null) {
                                int pitch = (int) (2 - Math.floor(arena.timer / 2.0));
                                world.playSound(arena.lobbyWarp, Sound.ENTITY_EXPERIENCE_ORB_PICKUP, 1, pitch);
                            }
                            ArenaHandler.sendFMessage(arena, MessageKey.NORMAL_LOBBY_ARENA_IS_STARTING, "1-" + arena.timer);
                        }
                    } else {
                        arena.gameState = ArenaState.IN_GAME;
                        arena.timer = arena.gameTime;
                        ArenaHandler.sendFMessage(arena, MessageKey.NORMAL_LOBBY_ARENA_STARTED, "secs-" + arena.waitingTimeSeeker);

                        for (int i = arena.amountSeekersOnStart; i > 0; i = i - 1) {
                            boolean loop = true;
                            Player seeker = arena.playersInArena.get(MemoryStorage.random.nextInt(arena.playersInArena.size()));

                            for (Player playerCheck : arena.playersInArena) {
                                if (MemoryStorage.chosenSeeker.get(playerCheck) != null) {
                                    if (MemoryStorage.chosenSeeker.get(playerCheck)) {
                                        seeker = playerCheck;
                                        MemoryStorage.chosenSeeker.remove(playerCheck);
                                    } else {
                                        if (seeker.equals(playerCheck)) {
                                            i = i + 1;
                                            loop = false;
                                        }
                                    }
                                }
                            }

                            if (loop) {
                                if (!arena.seekers.contains(seeker)) {
                                    ArenaHandler.sendFMessage(arena, MessageKey.NORMAL_IN_GAME_SEEKER_CHOSEN, "seeker-" + seeker.getName());
                                    arena.seekers.add(seeker);
                                    PlayerHandler.teleport(seeker, arena.seekersWarp);
                                    seeker.getInventory().clear();
                                    seeker.updateInventory();
                                    seeker.setWalkSpeed(0.3F);
                                    MemoryStorage.seekertime.put(seeker, arena.waitingTimeSeeker);
                                } else {
                                    i = i + 1;
                                }
                            }
                        }

                        for (Player arenaPlayer : arena.playersInArena) {
                            if (!arena.seekers.contains(arenaPlayer)) {
                                arenaPlayer.getInventory().clear();
                                arenaPlayer.updateInventory();
                                ItemStack block = arena.disguiseBlocks.get(MemoryStorage.random.nextInt(arena.disguiseBlocks.size()));

                                if (MemoryStorage.chosenBlock.get(arenaPlayer) != null) {
                                    block = MemoryStorage.chosenBlock.get(arenaPlayer);
                                    MemoryStorage.chosenBlock.remove(arenaPlayer);
                                }

                                MiscDisguise disguise = new MiscDisguise(DisguiseType.FALLING_BLOCK, block.getType());
                                DisguiseAPI.disguiseToAll(arenaPlayer, disguise);
                                PlayerHandler.teleport(arenaPlayer, arena.hidersWarp);
                                ItemStack blockCount = new ItemStack(block.getType(), 5);
                                arenaPlayer.getInventory().setItem(8, blockCount);
                                arenaPlayer.getInventory().setHelmet(new ItemStack(block));
                                MemoryStorage.pBlock.put(arenaPlayer, block);
                                String blockName = block.getType().name();
                                blockName = WordUtils.capitalizeFully(blockName.replace("_", " "));
                                MessageManager.sendFMessage(arenaPlayer, MessageKey.NORMAL_IN_GAME_BLOCK,
                                        "block-" + blockName);
                            }
                        }
                    }
                }

                for (Player player : arena.seekers) {
                    ItemStack item = player.getInventory().getItem(0);
                    if (item == null || item.getType() != Material.DIAMOND_SWORD) {
                        player.getInventory().clear(); // semi prevent duping infinite swords. TODO: Fix this properly
                        ItemStack i = new ItemStack(Material.DIAMOND_SWORD, 1);
                        i.addUnsafeEnchantment(Enchantment.DAMAGE_ALL, 10);
                        player.getInventory().setItem(0, i);
                        player.getInventory().setHelmet(new ItemStack(Material.IRON_HELMET, 1));
                        player.getInventory().setChestplate(new ItemStack(Material.IRON_CHESTPLATE, 1));
                        player.getInventory().setLeggings(new ItemStack(Material.IRON_LEGGINGS, 1));
                        player.getInventory().setBoots(new ItemStack(Material.IRON_BOOTS, 1));
                        ItemStack infBow = new ItemStack(Material.BOW, 1);
                        infBow.addEnchantment(Enchantment.ARROW_INFINITE, 1);
                        player.getInventory().setItem(1, infBow);
                        player.getInventory().setItem(2, new ItemStack(Material.ARROW, 1));
                        player.playSound(player.getLocation(), Sound.BLOCK_ANVIL_USE, 1, 1);
                    }

                    if (MemoryStorage.seekertime.get(player) != null) {
                        MemoryStorage.seekertime.put(player, MemoryStorage.seekertime.get(player) - 1);
                        if (MemoryStorage.seekertime.get(player) <= 0) {
                            PlayerHandler.teleport(player, arena.hidersWarp);
                            MemoryStorage.seekertime.remove(player);
                            ArenaHandler.sendFMessage(arena, MessageKey.NORMAL_IN_GAME_SEEKER_SPAWNED,
                                    "%playerName%-" + player.getName());
                        }
                    }
                }

                if (arena.gameState == ArenaState.IN_GAME) {
                    arena.timer = arena.timer - 1;
                    if (arena.timer > 0) {
                        if (arena.timer == arena.gameTime - arena.timeUntilHidersSword) {
                            ItemStack sword = new ItemStack(Material.WOODEN_SWORD, 1);
                            sword.addUnsafeEnchantment(Enchantment.KNOCKBACK, 2);
                            for (Player arenaPlayer : arena.playersInArena) {
                                if (!arena.seekers.contains(arenaPlayer)) {
                                    arenaPlayer.getInventory().addItem(sword);
                                    MessageManager.sendFMessage(arenaPlayer, MessageKey.NORMAL_IN_GAME_GIVEN_SWORD);
                                }
                            }
                        }

                        // blockAnnouncer code.
                        if ((arena.blockAnnouncerTime > 0) && (arena.timer == arena.blockAnnouncerTime)) {
                            ArrayList<String> remainingBlocks = new ArrayList<>();
                            for (Player arenaPlayer : arena.playersInArena) {
                                if (!arena.seekers.contains(arenaPlayer)) {
                                    ItemStack item = arenaPlayer.getInventory().getItem(8);
                                    String block = item == null ? Material.AIR.name() : item.getType().name();
                                    block = WordUtils.capitalizeFully(block.replace("_", " "));
                                    if (!remainingBlocks.contains(block)) { //Don't print double up block names.
                                        remainingBlocks.add(block);
                                    }
                                }
                            }
                            String blocklist = StringUtils.join(remainingBlocks, ", ");
                            ArenaHandler.sendFMessage(arena, MessageKey.NORMAL_IN_GAME_BLOCKS_LEFT, "1-" + blocklist);
                        }

                        if (arena.timer == 190 || arena.timer == 60 || arena.timer == 30 || arena.timer == 10) {
                            ArenaHandler.sendFMessage(arena, MessageKey.NORMAL_IN_GAME_ARENA_END, "1-" + arena.timer);
                        } else if (arena.timer <= 5) {
                            World world = arena.lobbyWarp.getWorld();
                            if (world != null) {
                                int pitch = (int) (2 - Math.floor(arena.timer / 2.0));
                                world.playSound(arena.lobbyWarp, Sound.ENTITY_EXPERIENCE_ORB_PICKUP, 1, pitch);
                            }
                            ArenaHandler.sendFMessage(arena, MessageKey.NORMAL_LOBBY_ARENA_IS_STARTING, "1-" + arena.timer);
                        }
                    } else {
                        ArenaHandler.hidersWin(arena);
                        return;
                    }

                    for (Player player : arena.playersInArena) {
                        if (!arena.seekers.contains(player)) {
                            Location pLoc = player.getLocation();
                            Location moveLoc = MemoryStorage.moveLoc.get(player);
                            ItemStack block = player.getInventory().getItem(8);

                            if (block == null) {
                                if (MemoryStorage.pBlock.get(player) != null) {
                                    block = MemoryStorage.pBlock.get(player);
                                    player.getInventory().setItem(8, block);
                                    player.updateInventory();
                                }
                            }

                            if (moveLoc != null && block != null) {
                                if (moveLoc.getX() == pLoc.getX() && moveLoc.getY() == pLoc.getY() &&
                                        moveLoc.getZ() == pLoc.getZ()) {
                                    if (block.getAmount() > 1) {
                                        block.setAmount(block.getAmount() - 1);
                                    } else {
                                        Block pBlock = player.getLocation().getBlock();
                                        BlockData pBlockData = pBlock.getBlockData();
                                        if (pBlockData.getMaterial().equals(Material.AIR) || pBlockData.getMaterial().equals(Material.WATER)
                                                || pBlockData.getMaterial().equals(Material.WATER)) {
                                            if (pBlockData.getMaterial().equals(Material.WATER) || pBlockData.getMaterial().equals(Material.WATER)) {
                                                MemoryStorage.hiddenLocWater.put(player, true);
                                            } else {
                                                MemoryStorage.hiddenLocWater.put(player, false);
                                            }
                                            if (DisguiseAPI.isDisguised(player)) {
                                                DisguiseAPI.undisguiseToAll(player);
                                                for (Player pl : Bukkit.getOnlinePlayers()) {
                                                    if (!pl.equals(player)) {
                                                        pl.hidePlayer(this, player);
                                                        pl.sendBlockChange(pBlock.getLocation(), block.getType().createBlockData());
                                                    }
                                                }

                                                block.addUnsafeEnchantment(Enchantment.DURABILITY, 10);
                                                player.playSound(pLoc, Sound.ENTITY_EXPERIENCE_ORB_PICKUP, 1, 1);
                                                MemoryStorage.hiddenLoc.put(player, moveLoc);
                                                String blockName = block.getType().name();
                                                blockName = WordUtils.capitalizeFully(blockName.replace("_", " "));
                                                MessageManager.sendFMessage(player, MessageKey.NORMAL_IN_GAME_NOW_SOLID, "block-"
                                                        + blockName);
                                            }
                                            for (Player pl : Bukkit.getOnlinePlayers()) {
                                                if (!pl.equals(player)) {
                                                    pl.hidePlayer(this, player);
                                                    pl.sendBlockChange(pBlock.getLocation(), block.getType().createBlockData());
                                                }
                                            }
                                        } else {
                                            MessageManager.sendFMessage(player, MessageKey.WARNING_IN_GAME_NO_SOLID_PLACE);
                                        }
                                    }
                                } else {
                                    block.setAmount(5);
                                    if (!DisguiseAPI.isDisguised(player)) {
                                        SolidBlockHandler.makePlayerUnsolid(player);
                                    }
                                }
                            }
                        }
                    }
                    ScoreboardHandler.updateScoreboard(arena); // TODO Only do this when needed (player added/removed)
                }

                for (Player pl : arena.playersInArena) {
                    pl.setLevel(arena.timer);
                    if (arena.seekers.contains(pl)) {
                        pl.setGameMode(GameMode.SURVIVAL);
                    } else {
                        pl.setGameMode(GameMode.ADVENTURE);
                    }
                }
            }
            SignsHandler.updateSigns(); //TODO Only do this when needed (game-state change or player count change)
        }, 0, 20);
    }

    public void onDisable() {
        for (Arena arena : MemoryStorage.arenaList) {
            String cause = "[BlockHunt] Arena " + arena.arenaName + " has been stopped";
            ArenaHandler.stopArena(arena, cause, MessageKey.WARNING_ARENA_STOPPED);
        }

        MessageManager.sendFMessage(null, MessageKey.LOG_DISABLED_PLUGIN, "name-" + BlockHunt.pluginDescriptionFile.getName(), "version-" + BlockHunt.pluginDescriptionFile.getVersion(), "authors-"
                + BlockHunt.pluginDescriptionFile.getAuthors().get(0));
    }

    public static PluginDescriptionFile getPluginDescriptionFile() {
        return pluginDescriptionFile;
    }

    /**
     * Short a String for like the Scoreboard title.
     *
     * @param string    String to be shortened.
     * @param maxLength Max length of the characters.
     * @return Shorten string, else normal string.
     */
    public static String cutString(String string, int maxLength) {
        if (string.length() > maxLength) {
            string = string.substring(0, maxLength);
        }
        return string;
    }

}
