package net.knarcraft.blockhunt.manager;

import net.knarcraft.blockhunt.BlockHunt;
import net.knarcraft.blockhunt.MemoryStorage;
import net.knarcraft.blockhunt.command.BlockHuntCommand;
import net.knarcraft.blockhunt.command.BlockHuntTabCompleter;
import net.knarcraft.blockhunt.command.CreateCommand;
import net.knarcraft.blockhunt.command.HelpCommand;
import net.knarcraft.blockhunt.command.InfoCommand;
import net.knarcraft.blockhunt.command.JoinCommand;
import net.knarcraft.blockhunt.command.LeaveCommand;
import net.knarcraft.blockhunt.command.ListCommand;
import net.knarcraft.blockhunt.command.ReloadCommand;
import net.knarcraft.blockhunt.command.RemoveCommand;
import net.knarcraft.blockhunt.command.SetCommand;
import net.knarcraft.blockhunt.command.SetWarpCommand;
import net.knarcraft.blockhunt.command.ShopCommand;
import net.knarcraft.blockhunt.command.StartCommand;
import net.knarcraft.blockhunt.command.TokensCommand;
import net.knarcraft.blockhunt.command.WandCommand;
import net.knarcraft.blockhunt.config.ConfigKey;
import net.knarcraft.blockhunt.config.MessageKey;
import net.knarcraft.blockhunt.config.Permission;
import org.bukkit.command.CommandExecutor;
import org.bukkit.command.PluginCommand;
import org.bukkit.command.TabCompleter;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;

public class CommandManager {

    private static Command blockHuntCommand;
    private static Command helpCommand;
    private static Command joinCommand;
    private static Command startCommand;
    private static Command createCommand;
    private static Command setCommand;
    private static Command setWarpCommand;
    private static Command removeCommand;
    private static Command tokensCommand;

    private CommandManager() {

    }

    public static void load() {
        if (blockHuntCommand != null) {
            return;
        }
        List<String> BlockHuntCMD = new ArrayList<>() {{
            add("info");
            add("help");
            add("reload");
            add("join");
            add("leave");
            add("list");
            add("shop");
            add("start");
            add("wand");
            add("create");
            add("set");
            add("setWarp");
            add("remove");
            add("tokens");
        }};
        blockHuntCommand = new Command("BlockHunt", null, null, Permission.INFO,
                MessageKey.HELP_INFO, (Boolean) MemoryStorage.config.get(ConfigKey.COMMAND_ENABLED_INFO), BlockHuntCMD,
                new InfoCommand(), null);
        MemoryStorage.commands.add(blockHuntCommand);
        MemoryStorage.commands.add(new Command("BlockHunt INFO", "info", "i",
                Permission.INFO, MessageKey.HELP_INFO, (Boolean) MemoryStorage.config.get(ConfigKey.COMMAND_ENABLED_INFO),
                BlockHuntCMD, new InfoCommand(), "/BlockHunt [info|i]"));
        helpCommand = new Command("BlockHunt HELP", "help", "h",
                Permission.HELP, MessageKey.HELP_HELP, (Boolean) MemoryStorage.config.get(ConfigKey.COMMAND_ENABLED_HELP),
                BlockHuntCMD, new HelpCommand(), "/BlockHunt <help|h> [page number]");
        MemoryStorage.commands.add(helpCommand);
        MemoryStorage.commands.add(new Command("BlockHunt RELOAD", "reload", "r",
                Permission.RELOAD, MessageKey.HELP_RELOAD,
                (Boolean) MemoryStorage.config.get(ConfigKey.COMMAND_ENABLED_RELOAD), BlockHuntCMD,
                new ReloadCommand(), "/BlockHunt <reload|r>"));
        joinCommand = new Command("BlockHunt JOIN", "join", "j",
                Permission.JOIN, MessageKey.HELP_JOIN, (Boolean) MemoryStorage.config.get(ConfigKey.COMMAND_ENABLED_JOIN),
                BlockHuntCMD, new JoinCommand(), "/BlockHunt <join|j> <arena name>");
        MemoryStorage.commands.add(joinCommand);
        MemoryStorage.commands.add(new Command("BlockHunt LEAVE", "leave", "l",
                Permission.LEAVE, MessageKey.HELP_LEAVE, (Boolean) MemoryStorage.config.get(
                ConfigKey.COMMAND_ENABLED_LEAVE), BlockHuntCMD, new LeaveCommand(), "/BlockHunt <leave|l>"));
        MemoryStorage.commands.add(new Command("BlockHunt LIST", "list", "li",
                Permission.LIST, MessageKey.HELP_LIST, (Boolean) MemoryStorage.config.get(ConfigKey.COMMAND_ENABLED_LIST),
                BlockHuntCMD, new ListCommand(), "/BlockHunt <list|li>"));
        MemoryStorage.commands.add(new Command("BlockHunt SHOP", "shop", "sh",
                Permission.SHOP, MessageKey.HELP_SHOP, (Boolean) MemoryStorage.config.get(ConfigKey.COMMAND_ENABLED_SHOP),
                BlockHuntCMD, new ShopCommand(), "/BlockHunt <shop|sh>"));
        startCommand = new Command("BlockHunt START", "start", "go",
                Permission.START, MessageKey.HELP_START,
                (Boolean) MemoryStorage.config.get(ConfigKey.COMMAND_ENABLED_START), BlockHuntCMD, new StartCommand(),
                "/BlockHunt <start|go> <arena name>");
        MemoryStorage.commands.add(startCommand);
        MemoryStorage.commands.add(new Command("BlockHunt WAND", "wand", "w",
                Permission.CREATE, MessageKey.HELP_WAND, (Boolean) MemoryStorage.config.get(ConfigKey.COMMAND_ENABLED_WAND),
                BlockHuntCMD, new WandCommand(), "/BlockHunt <wand|w>"));
        createCommand = new Command("BlockHunt CREATE", "create", "c",
                Permission.CREATE, MessageKey.HELP_CREATE,
                (Boolean) MemoryStorage.config.get(ConfigKey.COMMAND_ENABLED_CREATE), BlockHuntCMD, new CreateCommand(),
                "/BlockHunt <create|c> <arena name>");
        MemoryStorage.commands.add(createCommand);
        setCommand = new Command("BlockHunt SET", "set", "s",
                Permission.SET, MessageKey.HELP_SET, (Boolean) MemoryStorage.config.get(ConfigKey.COMMAND_ENABLED_SET),
                BlockHuntCMD, new SetCommand(), "/BlockHunt <set|s> <arena name>");
        MemoryStorage.commands.add(setCommand);
        setWarpCommand = new Command("BlockHunt SETWARP", "setWarp", "sw",
                Permission.SET_WARP, MessageKey.HELP_SET_WARP,
                (Boolean) MemoryStorage.config.get(ConfigKey.COMMAND_ENABLED_SET_WARP), BlockHuntCMD,
                new SetWarpCommand(), "/BlockHunt <setWarp|sw> <lobby|hiders|seekers|spawn> <arena name>");
        MemoryStorage.commands.add(setWarpCommand);
        removeCommand = new Command("BlockHunt REMOVE", "remove", "delete",
                Permission.REMOVE, MessageKey.HELP_REMOVE,
                (Boolean) MemoryStorage.config.get(ConfigKey.COMMAND_ENABLED_REMOVE), BlockHuntCMD, new RemoveCommand(),
                "/BlockHunt <remove|delete> <arena name>");
        MemoryStorage.commands.add(removeCommand);
        tokensCommand = new Command("BlockHunt TOKENS", "tokens", "t",
                Permission.TOKENS, MessageKey.HELP_TOKENS,
                (Boolean) MemoryStorage.config.get(ConfigKey.COMMAND_ENABLED_TOKENS), BlockHuntCMD, new TokensCommand(),
                "/BlockHunt <tokens|t> <set|add|take> <player name> <amount>");
        MemoryStorage.commands.add(tokensCommand);
        registerCommand(new BlockHuntCommand(), new BlockHuntTabCompleter());
    }

    public static Command getRemoveCommand() {
        return removeCommand;
    }

    public static Command getSetCommand() {
        return setCommand;
    }

    public static Command getCreateCommand() {
        return createCommand;
    }

    public static Command getStartCommand() {
        return startCommand;
    }

    public static Command getHelpCommand() {
        return helpCommand;
    }

    public static Command getTokensCommand() {
        return tokensCommand;
    }

    public static Command getJoinCommand() {
        return joinCommand;
    }

    public static Command setWarpCommand() {
        return setWarpCommand;
    }

    /**
     * Registers a command
     *
     * @param commandExecutor <p>The executor for the command</p>
     * @param tabCompleter    <p>The tab-completer to use, or null</p>
     */
    private static void registerCommand(@NotNull CommandExecutor commandExecutor, @Nullable TabCompleter tabCompleter) {
        PluginCommand command = BlockHunt.plugin.getCommand("BlockHunt");
        if (command != null) {
            command.setExecutor(commandExecutor);
            if (tabCompleter != null) {
                command.setTabCompleter(tabCompleter);
            }
        } else {
            BlockHunt.plugin.getLogger().log(Level.SEVERE, "Unable to register the command " + "BlockHunt");
        }
    }

}
